<?php

namespace Teknisa\TCO\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\Report\ReportService;

class Tco01009AnaliseCustos extends \Zeedhi\Framework\Controller\Report {
    
    protected $lista;
    protected $mensagem;
    protected $novoCodigo;
    protected $crudService;
    protected $environment;
    protected $analiseCustosService;
    protected $entityManager;
    protected $birtStrategy;

    public function __construct($reportService, $environment, $analiseCustosService, $lista, $novoCodigo, $mensagem, $entityManager, $birtStrategy) {
        $this->reportService        = $reportService;
        $this->lista                = $lista;
        $this->mensagem             = $mensagem;
        $this->novoCodigo           = $novoCodigo;

        $this->environment          = $environment;
        $this->analiseCustosService = $analiseCustosService;
        $this->entityManager        = $entityManager;
        $this->birtStrategy         = $birtStrategy;
    }
    
    public function executaProcedureAnaliseCusto(DTO\Request $request, DTO\Response $response) {
        $this->entityManager->beginTransaction();
        $row = $request->getRow();
        try {
            $row["IDORIGEM"] = "SIMULADOR";
            $row['L_NRCONFIGCARDBASE'] = $this->createLista($row['NRCONFIGCARDBASE']);
            $row['L_PORCGRAMCARDBASE'] = $this->createLista($row['PORCGRAMCARDBASE']);
            
            $callback = $this->analiseCustosService->procedureAnaliseCusto($row);
            
            if(!empty($callback['P_LISTAERROS'])){
                $struct = array(
                    'TIPO' => 'LISTA',
                    'CDFILIAL' => $this->environment->getCdFilial(),
                    'NRLISTA'  => $callback['P_LISTAERROS']
                );  
                $response->addDataSet(new DataSet("ERRO", $this->mensagem->convertMessage($struct)));
                $this->entityManager->rollback();
            }else{
                $this->entityManager->commit();
                if ($row['IDCOMPLEMENTAR'] == 'S') {
                    $row['NRCARDTRABCOMPLEMENTADO'] = $callback['P_NRCARDAPIOTRAB'];
                    
                    $newCallback = $this->analiseCustosService->procedureAnaliseCustoServComplementar($row);
                    if(!empty($newCallback['P_LISTAERROS'])){
                        $struct = array(
                            'TIPO' => 'LISTA',
                            'CDFILIAL' => $this->environment->getCdFilial(),
                            'NRLISTA'  => $newCallback['P_LISTAERROS']
                        );  
                        $response->addDataSet(new DataSet("ERRO", $this->mensagem->convertMessage($struct)));
                    } else {
                        $response->addDataSet(new DataSet("RESPONSE", $newCallback));
                    }
                } else {
                    $response->addDataSet(new DataSet("RESPONSE", $callback));
                }
            }
        }catch(\Exception $e){
            $this->entityManager->rollback();
            throw $e;
        }
        
    }
    
    private function createLista($dados) {
        $nrlista = $this->novoCodigo->getCodeByTableName('LISTA', $this->environment->getNrorgTrab(), 1, 10, false);
        
        foreach($dados as $data) {
            $this->lista->insertLista($nrlista, $data);
        }
        
        return $nrlista;
    }
    
    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
        
        $row = $request->getRow();
        $erros = [];
        $nrConfig = [];
        $qtdErros = 0;

        try{
            $nrlista = $this->novoCodigo->getCodeByTableName('LISTA', $this->environment->getNrorgTrab(), 1, 10, false);
            $row["IDORIGEM"] = "SIMULADOR";
            
            $row['L_NRCONFIGCARDBASE']  = $this->createLista($row['NRCONFIGCARDBASE_PROCEDURE']);
            $row['L_PORCGRAMCARDBASE']  = $this->createLista($row['PORCGRAMCARDBASE_PROCEDURE']);
            $row['NRCONFIGCARDBASE']    = isset($row['NMTIPOPRAT']) && ($row['NMTIPOPRAT'] != "") && sizeof($row['NMTIPOPRAT']) > 0? '_NRLISTA='.$this->createLista($row['NMTIPOPRAT']): 'T';
            $row['NRSEMANA_INICIAL']    = isset($row['NRSEMANA_INICIAL']) ? $row['NRSEMANA_INICIAL']: null;
            $row['NRDIASEMANA_INICIAL'] = isset($row['NRDIASEMANA_INICIAL']) ? $row['NRDIASEMANA_INICIAL']: null;
            $row['NRSEMANA_FINAL']      = isset($row['NRSEMANA_FINAL'])? $row['NRSEMANA_FINAL']: null;
            $row['NRDIASEMANA_FINAL']   = isset($row['NRDIASEMANA_FINAL'])? $row['NRDIASEMANA_FINAL']: null;
            
            
            if(!isset($row['P_NRLISTA']) || !$row['P_NRLISTA']){
                
                foreach ($row['REGIOES'] as $regiao) {
                    
                    $this->entityManager->beginTransaction();
                    
                    $row['CDREGIAO'] = $regiao;
                    $callback = $this->analiseCustosService->procedureAnaliseCusto($row);
                    
                    if(!empty($callback['P_LISTAERROS'])){
                        
                        $struct = array(
                            'TIPO' => 'LISTA',
                            'CDFILIAL' => $this->environment->getCdFilial(),
                            'NRLISTA'  => $callback['P_LISTAERROS']
                        );  
                        $prefixo = array('MSGCODE'=> 'TCO01009REGIAO', 'PARAMS'=> array('CDREGIAO' => $row['CDREGIAO']));
                        array_push($erros, $prefixo);
                        $erros = array_merge($erros, $this->mensagem->convertMessage($struct));
                        $qtdErros++;
                        
                        $this->entityManager->rollback();
                        
                    }else{
                        
                        $this->entityManager->commit();
                        
                        if ($row['IDCOMPLEMENTAR'] == 'S') {
                            
                            $row['NRCARDTRABCOMPLEMENTADO'] = $callback['P_NRCARDAPIOTRAB'];
                            $newCallback = $this->analiseCustosService->procedureAnaliseCustoServComplementar($row);
                            
                            if(!empty($newCallback['P_LISTAERROS'])){
                                
                                $struct = array(
                                    'TIPO' => 'LISTA',
                                    'CDFILIAL' => $this->environment->getCdFilial(),
                                    'NRLISTA'  => $newCallback['P_LISTAERROS']
                                ); 
                                $prefixo = array('MSGCODE'=> 'Região - '.$row['CDREGIAO'].':', 'PARAMS'=> array());
                                array_push($erros, $prefixo);
                                $erros = array_merge($erros, $this->mensagem->convertMessage($struct));
                                $qtdErros++;
                                
                            } else {
                                
                                $this->lista->insertLista($nrlista, $newCallback['P_NRCARDAPIOTRAB']);
                                array_push($nrConfig, $newCallback['P_NRCARDAPIOTRAB']);
                                
                            }
                        } else {
                            
                            $this->lista->insertLista($nrlista, $callback['P_NRCARDAPIOTRAB']);
                            array_push($nrConfig, $callback['P_NRCARDAPIOTRAB']);
                            
                        }
                    }
                }
                
                if($qtdErros > 0){
                    $response->addDataSet(new DataSet("ERROS", $erros));
                }
                $row['P_NRLISTA'] = $nrlista;
            }
            $row['NRORG'] = $this->environment->getNrorgTrab();
            $this->birtStrategy->setReportFormat($row['QR2TYPE']);
            $row['__report_name'] = $row['REPORT_NAME'];
            
            $response->addDataSet(new DataSet("RESPONSE", array('NRLISTA' => $row['P_NRLISTA'])));
        }catch(\Exception $e){
            
            throw $e;
        }
    }
    
    public function apagaCardapioTrab($request, $response){
        $row = $request->getRow();
        try{
            $this->analiseCustosService->apagaCardapioTrab($row);
        }catch(\Exception $e){
            throw $e;
        }
    }
    
    protected $reportMapping = array ( // To-Do
        'TCO01000' =>
            array (
                'strategy' => 'BIRT',
                'parameterMapping' =>
                    array (
                        'QTSEGSEX'           => 'P_QTSEGSEX',
                        'QTSABADO'           => 'P_QTSABADO',
                        'QTDOMFERI'          => 'P_QTDOMFERI',
                        'NRPADRAOQUALIDADE'  => 'P_NRPADRAOQUALIDADE',
                        'NRTIPOPUBLICO'      => 'P_NRTIPOPUBLICO',
                        'L_NRCONFIGCARDBASE' => 'P_CONFIGURACOES',
                        'IDIOMA'             => 'P_IDIOMA',
                        'L_PORCGRAMCARDBASE' => 'P_GRUPOS',
                        'NRATIVCLISEGMER'    => 'P_NRATIVCLISEGMER',
                        'P_NRLISTA'          => 'P_NRLISTA',
                        'NRORG'              => 'P_NRORG'
                    )
            ),
        'TCO01001' =>
            array (
                'strategy' => 'BIRT',
                'parameterMapping' =>
                    array (
                        'QTSEGSEX'           => 'P_QTSEGSEX',
                        'QTSABADO'           => 'P_QTSABADO',
                        'QTDOMFERI'          => 'P_QTDOMFERI',
                        'NRPADRAOQUALIDADE'  => 'P_NRPADRAOQUALIDADE',
                        'NRTIPOPUBLICO'      => 'P_NRTIPOPUBLICO',
                        'L_NRCONFIGCARDBASE' => 'P_CONFIGURACOES',
                        'IDIOMA'             => 'P_IDIOMA',
                        'L_PORCGRAMCARDBASE' => 'P_GRUPOS',
                        'NRATIVCLISEGMER'    => 'P_NRATIVCLISEGMER',
                        'P_NRLISTA'          => 'P_NRLISTA',
                        'NRORG'              => 'P_NRORG'
                    )
            ),
        'TCO01002' =>
            array (
                'strategy' => 'BIRT',
                'parameterMapping' =>
                    array (
                        'QTSEGSEX'           => 'P_QTSEGSEX',
                        'QTSABADO'           => 'P_QTSABADO',
                        'QTDOMFERI'          => 'P_QTDOMFERI',
                        'NRPADRAOQUALIDADE'  => 'P_NRPADRAOQUALIDADE',
                        'NRTIPOPUBLICO'      => 'P_NRTIPOPUBLICO',
                        'L_NRCONFIGCARDBASE' => 'P_CONFIGURACOES',
                        'IDIOMA'             => 'P_IDIOMA',
                        'L_PORCGRAMCARDBASE' => 'P_GRUPOS',
                        'NRATIVCLISEGMER'    => 'P_NRATIVCLISEGMER',
                        'P_NRLISTA'          => 'P_NRLISTA',
                        'NRORG'              => 'P_NRORG'
                    )
            ),
        'TCO01003' =>
            array (
                'strategy' => 'BIRT',
                'parameterMapping' =>
                    array (
                        'QTSEGSEX'           => 'P_QTSEGSEX',
                        'QTSABADO'           => 'P_QTSABADO',
                        'QTDOMFERI'          => 'P_QTDOMFERI',
                        'NRPADRAOQUALIDADE'  => 'P_NRPADRAOQUALIDADE',
                        'NRTIPOPUBLICO'      => 'P_NRTIPOPUBLICO',
                        'L_NRCONFIGCARDBASE' => 'P_CONFIGURACOES',
                        'IDIOMA'             => 'P_IDIOMA',
                        'L_PORCGRAMCARDBASE' => 'P_GRUPOS',
                        'NRATIVCLISEGMER'    => 'P_NRATIVCLISEGMER',
                        'P_NRLISTA'          => 'P_NRLISTA',
                        'NRORG'              => 'P_NRORG'
                    )
            ),
        'TCO01004' =>
            array (
                'strategy' => 'BIRT',
                'parameterMapping' =>
                    array (
                        'QTSEGSEX'            => 'P_QTSEGSEX',  
                        'QTSABADO'            => 'P_QTSABADO',  
                        'QTDOMFERI'           => 'P_QTDOMFERI',  
                        'NRPADRAOQUALIDADE'   => 'P_NRPADRAOQUALIDADE',  
                        'NRTIPOPUBLICO'       => 'P_NRTIPOPUBLICO',  
                        'L_NRCONFIGCARDBASE'  => 'P_CONFIGURACOES',  
                        'IDIOMA'              => 'P_IDIOMA',  
                        'L_PORCGRAMCARDBASE'  => 'P_GRUPOS',  
                        'NRATIVCLISEGMER'     => 'P_NRATIVCLISEGMER',  
                        'P_NRLISTA'           => 'P_NRLISTA',  
                        'VRPEACRESCUSTO'      => 'P_VRPEACRESCUSTO', 
                        'NRCONFIGCARDBASE'    => 'P_NRCONFIGCARDBASE', 
                        'NRSEMANA_INICIAL'    => 'P_NRSEMANA_INICIAL', 
                        'NRDIASEMANA_INICIAL' => 'P_NRDIASEMANA_INICIAL', 
                        'NRSEMANA_FINAL'      => 'P_NRSEMANA_FINAL', 
                        'NRDIASEMANA_FINAL'   => 'P_NRDIASEMANA_FINAL', 
                        'CDARVPRODINI'        => 'P_CDARVPRODINI', 
                        'CDARVPRODFIN'        => 'P_CDARVPRODFIN', 
                        'CDPRATOINI'          => 'P_CDPRATOINI', 
                        'CDPRATOFIM'          => 'P_CDPRATOFIM', 
                        'NRORG'               => 'P_NRORG'
                    )
            ),
        'TCO01005' =>
            array (
                'strategy' => 'BIRT',
                'parameterMapping' =>
                    array (
                        'QTSEGSEX'            => 'P_QTSEGSEX',  
                        'QTSABADO'            => 'P_QTSABADO',  
                        'QTDOMFERI'           => 'P_QTDOMFERI',  
                        'NRPADRAOQUALIDADE'   => 'P_NRPADRAOQUALIDADE',  
                        'NRTIPOPUBLICO'       => 'P_NRTIPOPUBLICO',  
                        'L_NRCONFIGCARDBASE'  => 'P_CONFIGURACOES',  
                        'IDIOMA'              => 'P_IDIOMA',  
                        'L_PORCGRAMCARDBASE'  => 'P_GRUPOS',  
                        'NRATIVCLISEGMER'     => 'P_NRATIVCLISEGMER',  
                        'P_NRLISTA'           => 'P_NRLISTA',  
                        'VRPEACRESCUSTO'      => 'P_VRPEACRESCUSTO', 
                        'NRCONFIGCARDBASE'    => 'P_NRCONFIGCARDBASE', 
                        'NRSEMANA_INICIAL'    => 'P_NRSEMANA_INICIAL', 
                        'NRDIASEMANA_INICIAL' => 'P_NRDIASEMANA_INICIAL', 
                        'NRSEMANA_FINAL'      => 'P_NRSEMANA_FINAL', 
                        'NRDIASEMANA_FINAL'   => 'P_NRDIASEMANA_FINAL', 
                        'CDARVPRODINI'        => 'P_CDARVPRODINI', 
                        'CDARVPRODFIN'        => 'P_CDARVPRODFIN', 
                        'CDPRATOINI'          => 'P_CDPRATOINI', 
                        'CDPRATOFIM'          => 'P_CDPRATOFIM', 
                        'NRORG'               => 'P_NRORG'
                    )
            ),
        'TCO01006' =>
            array (
                'strategy' => 'BIRT',
                'parameterMapping' =>
                    array (
                        'QTSEGSEX'            => 'P_QTSEGSEX',  
                        'QTSABADO'            => 'P_QTSABADO',  
                        'QTDOMFERI'           => 'P_QTDOMFERI',  
                        'NRPADRAOQUALIDADE'   => 'P_NRPADRAOQUALIDADE',  
                        'NRTIPOPUBLICO'       => 'P_NRTIPOPUBLICO',  
                        'L_NRCONFIGCARDBASE'  => 'P_CONFIGURACOES',  
                        'IDIOMA'              => 'P_IDIOMA',  
                        'L_PORCGRAMCARDBASE'  => 'P_GRUPOS',  
                        'NRATIVCLISEGMER'     => 'P_NRATIVCLISEGMER',  
                        'P_NRLISTA'           => 'P_NRLISTA',  
                        'VRPEACRESCUSTO'      => 'P_VRPEACRESCUSTO', 
                        'NRCONFIGCARDBASE'    => 'P_NRCONFIGCARDBASE', 
                        'NRSEMANA_INICIAL'    => 'P_NRSEMANA_INICIAL', 
                        'NRDIASEMANA_INICIAL' => 'P_NRDIASEMANA_INICIAL', 
                        'NRSEMANA_FINAL'      => 'P_NRSEMANA_FINAL', 
                        'NRDIASEMANA_FINAL'   => 'P_NRDIASEMANA_FINAL', 
                        'CDARVPRODINI'        => 'P_CDARVPRODINI', 
                        'CDARVPRODFIN'        => 'P_CDARVPRODFIN', 
                        'CDPRATOINI'          => 'P_CDPRATOINI', 
                        'CDPRATOFIM'          => 'P_CDPRATOFIM', 
                        'NRORG'               => 'P_NRORG'
                    )
            )
    );
    
    public function vericaGrupoTipoPrato($request, $response){
        $row = $request->getRow();
        try{
           $result = $this->analiseCustosService->vericaGrupoTipoPrato($row);
           $response->addDataSet(new DataSet("GruposTipoPrato", $result));
        }catch(\Exception $e){
            throw $e;
        }
    }
}





